﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  [StorableClass]
  public class PercentValueRange : Range<PercentValue> {
    #region Constructors and Cloning
    [StorableConstructor]
    protected PercentValueRange(bool deserializing) : base(deserializing) { }
    protected PercentValueRange(PercentValueRange original, Cloner cloner) : base(original, cloner) { }
    public PercentValueRange(PercentValue lowerBound, PercentValue upperBound, PercentValue stepSize) : base(lowerBound, upperBound, stepSize) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PercentValueRange(this, cloner);
    }
    #endregion

    protected override PercentValue GetRandomSample(IRandom random) {
      double val;
      do {
        val = Math.Round((random.NextDouble() * (UpperBound.Value - LowerBound.Value) + LowerBound.Value) / StepSize.Value, 0) * StepSize.Value;
      } while (!IsInRange(val));
      return new PercentValue(val);
    }

    public DoubleValueRange AsDoubleValueRange() {
      return new DoubleValueRange(LowerBound, UpperBound, StepSize);
    }

    public double ApplyStepSize(double value) {
      return ((int)Math.Round(value / this.StepSize.Value, 0)) * this.StepSize.Value;
    }

    public bool IsInRange(double value) {
      return value <= this.UpperBound.Value && value >= this.LowerBound.Value;
    }

    public override IEnumerable<PercentValue> GetCombinations() {
      var solutions = new List<PercentValue>();
      double value = LowerBound.Value;

      while (value <= UpperBound.Value) {
        solutions.Add(new PercentValue(value));
        value += StepSize.Value;
        value = ApplyStepSize(value);
      }
      return solutions;
    }

    protected override double CalculateSimilarityValue(PercentValue a, PercentValue b) {
      double range = UpperBound.Value - LowerBound.Value;
      double diff = Math.Abs(a.Value - b.Value);
      return Math.Max(0, (range - (diff * 2)) / range);
    }
  }
}
